@@ -190,6 +190,16 @@ struct SSConfig {
     #[cfg(any(target_os = "linux", target_os = "android"))]
     outbound_fwmark: Option<u32>,
 
+    #[serde(skip_serializing_if = "Option::is_none")]
+    #[cfg(target_os = "freebsd")]
+    outbound_user_cookie: Option<u32>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    outbound_bind_addr: Option<String>,
+
+    #[serde(skip_serializing_if = "Option::is_none")]
+    outbound_bind_interface: Option<String>,
+
     #[serde(skip_serializing_if = "Option::is_none")]
     security: Option<SSSecurityConfig>,
 
@@ -264,6 +274,9 @@ struct SSLocalExtConfig {
     #[cfg(feature = "local-tun")]
     #[serde(skip_serializing_if = "Option::is_none")]
     tun_interface_address: Option<String>,
+    #[cfg(feature = "local-tun")]
+    #[serde(skip_serializing_if = "Option::is_none")]
+    tun_interface_destination: Option<String>,
     #[cfg(all(feature = "local-tun", unix))]
     #[serde(skip_serializing_if = "Option::is_none")]
     tun_device_fd_from_path: Option<String>,
@@ -856,6 +869,9 @@ pub struct LocalConfig {
     /// Tun interface's address and netmask
     #[cfg(feature = "local-tun")]
     pub tun_interface_address: Option<IpNet>,
+    /// Tun interface's destination address and netmask
+    #[cfg(feature = "local-tun")]
+    pub tun_interface_destination: Option<IpNet>,
     /// Tun interface's file descriptor
     #[cfg(all(feature = "local-tun", unix))]
     pub tun_device_fd: Option<std::os::unix::io::RawFd>,
@@ -899,6 +915,8 @@ impl LocalConfig {
             tun_interface_name: None,
             #[cfg(feature = "local-tun")]
             tun_interface_address: None,
+            #[cfg(feature = "local-tun")]
+            tun_interface_destination: None,
             #[cfg(all(feature = "local-tun", unix))]
             tun_device_fd: None,
             #[cfg(all(feature = "local-tun", unix))]
@@ -1706,8 +1724,8 @@ impl Config {
                     let mut user_manager = ServerUserManager::new();
 
                     for user in users {
-                        let key = match base64::decode_config(&user.password, base64::STANDARD) {
-                            Ok(k) => k,
+                        let user = match ServerUser::with_encoded_key(user.name, &user.password) {
+                            Ok(u) => u,
                             Err(..) => {
                                 let err = Error::new(
                                     ErrorKind::Malformed,
@@ -1718,7 +1736,7 @@ impl Config {
                             }
                         };
 
-                        user_manager.add_user(ServerUser::new(user.name, key));
+                        user_manager.add_user(user);
                     }
 
                     nsvr.set_user_manager(user_manager);
@@ -1924,6 +1942,26 @@ impl Config {
             nconfig.outbound_fwmark = Some(fwmark);
         }
 
+        // SO_USER_COOKIE
+        #[cfg(target_os = "freebsd")]
+        if let Some(user_cookie) = config.outbound_user_cookie {
+            nconfig.outbound_user_cookie = Some(user_cookie);
+        }
+
+        // Outbound bind() address
+        if let Some(bind_addr) = config.outbound_bind_addr {
+            match bind_addr.parse::<IpAddr>() {
+                Ok(b) => nconfig.outbound_bind_addr = Some(b),
+                Err(..) => {
+                    let err = Error::new(ErrorKind::Invalid, "invalid outbound_bind_addr", None);
+                    return Err(err);
+                }
+            }
+        }
+
+        // Bind device / interface
+        nconfig.outbound_bind_interface = config.outbound_bind_interface;
+
         // Security
         if let Some(sec) = config.security {
             if let Some(replay_attack) = sec.replay_attack {
@@ -2372,6 +2410,8 @@ impl fmt::Display for Config {
                         tun_interface_name: local.tun_interface_name.clone(),
                         #[cfg(feature = "local-tun")]
                         tun_interface_address: local.tun_interface_address.as_ref().map(ToString::to_string),
+                        #[cfg(feature = "local-tun")]
+                        tun_interface_destination: local.tun_interface_destination.as_ref().map(ToString::to_string),
                         #[cfg(all(feature = "local-tun", unix))]
                         tun_device_fd_from_path: local
                             .tun_device_fd_from_path
@@ -2457,7 +2497,7 @@ impl fmt::Display for Config {
                             for u in m.users_iter() {
                                 vu.push(SSServerUserConfig {
                                     name: u.name().to_owned(),
-                                    password: base64::encode(u.key()),
+                                    password: u.encoded_key(),
                                 });
                             }
                             vu
@@ -2581,6 +2621,14 @@ impl fmt::Display for Config {
             jconf.outbound_fwmark = self.outbound_fwmark;
         }
 
+        #[cfg(target_os = "freebsd")]
+        {
+            jconf.outbound_user_cookie = self.outbound_user_cookie;
+        }
+
+        jconf.outbound_bind_addr = self.outbound_bind_addr.map(|i| i.to_string());
+        jconf.outbound_bind_interface = self.outbound_bind_interface.clone();
+
         // Security
         if self.security.replay_attack.policy != ReplayAttackPolicy::default() {
             jconf.security = Some(SSSecurityConfig {
