@@ -12,11 +12,12 @@ use std::{
     time::Duration,
 };
 
-use base64::{decode_config, encode_config, URL_SAFE, URL_SAFE_NO_PAD};
+use base64::Engine as _;
 use byte_string::ByteStr;
 use bytes::Bytes;
 use cfg_if::cfg_if;
 use log::error;
+use thiserror::Error;
 use url::{self, Url};
 
 use crate::{
@@ -25,6 +26,27 @@ use crate::{
     relay::socks5::Address,
 };
 
+const USER_KEY_BASE64_ENGINE: base64::engine::GeneralPurpose = base64::engine::GeneralPurpose::new(
+    &base64::alphabet::STANDARD,
+    base64::engine::GeneralPurposeConfig::new()
+        .with_encode_padding(true)
+        .with_decode_padding_mode(base64::engine::DecodePaddingMode::Indifferent),
+);
+
+const AEAD2022_PASSWORD_BASE64_ENGINE: base64::engine::GeneralPurpose = base64::engine::GeneralPurpose::new(
+    &base64::alphabet::STANDARD,
+    base64::engine::GeneralPurposeConfig::new()
+        .with_encode_padding(true)
+        .with_decode_padding_mode(base64::engine::DecodePaddingMode::Indifferent),
+);
+
+const URL_PASSWORD_BASE64_ENGINE: base64::engine::GeneralPurpose = base64::engine::GeneralPurpose::new(
+    &base64::alphabet::URL_SAFE,
+    base64::engine::GeneralPurposeConfig::new()
+        .with_encode_padding(false)
+        .with_decode_padding_mode(base64::engine::DecodePaddingMode::Indifferent),
+);
+
 /// Shadowsocks server type
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum ServerType {
@@ -161,7 +183,7 @@ impl Debug for ServerUser {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.debug_struct("ServerUser")
             .field("name", &self.name)
-            .field("key", &base64::encode(&self.key))
+            .field("key", &USER_KEY_BASE64_ENGINE.encode(&self.key))
             .field("identity_hash", &ByteStr::new(&self.identity_hash))
             .finish()
     }
@@ -187,6 +209,15 @@ impl ServerUser {
         }
     }
 
+    /// Create a user from encoded key
+    pub fn with_encoded_key<N>(name: N, key: &str) -> Result<ServerUser, ServerUserError>
+    where
+        N: Into<String>,
+    {
+        let key = USER_KEY_BASE64_ENGINE.decode(key)?;
+        Ok(ServerUser::new(name, key))
+    }
+
     /// Name of the user
     pub fn name(&self) -> &str {
         self.name.as_str()
@@ -197,6 +228,11 @@ impl ServerUser {
         self.key.as_ref()
     }
 
+    /// Get Base64 encoded key of user
+    pub fn encoded_key(&self) -> String {
+        USER_KEY_BASE64_ENGINE.encode(&self.key)
+    }
+
     /// User's identity hash
     ///
     /// https://github.com/Shadowsocks-NET/shadowsocks-specs/blob/main/2022-2-shadowsocks-2022-extensible-identity-headers.md
@@ -212,6 +248,14 @@ impl ServerUser {
     }
 }
 
+/// ServerUser related errors
+#[derive(Debug, Clone, Error)]
+pub enum ServerUserError {
+    /// Invalid User key encoding
+    #[error("{0}")]
+    InvalidKeyEncoding(#[from] base64::DecodeError),
+}
+
 /// Server multi-users manager
 #[derive(Clone, Debug)]
 pub struct ServerUserManager {
@@ -302,7 +346,7 @@ pub struct ServerConfig {
 fn make_derived_key(method: CipherKind, password: &str, enc_key: &mut [u8]) {
     if method.is_aead_2022() {
         // AEAD 2022 password is a base64 form of enc_key
-        match base64::decode_config(password, base64::STANDARD) {
+        match AEAD2022_PASSWORD_BASE64_ENGINE.decode(password) {
             Ok(v) => {
                 if v.len() != enc_key.len() {
                     panic!(
@@ -363,7 +407,7 @@ where
         make_derived_key(method, upsk, &mut enc_key);
 
         for ipsk in split_iter {
-            match base64::decode_config(ipsk, base64::STANDARD) {
+            match USER_KEY_BASE64_ENGINE.decode(ipsk) {
                 Ok(v) => {
                     identity_keys.push(Bytes::from(v));
                 }
@@ -564,7 +608,7 @@ impl ServerConfig {
     /// ```
     pub fn to_qrcode_url(&self) -> String {
         let param = format!("{}:{}@{}", self.method(), self.password(), self.addr());
-        format!("ss://{}", encode_config(param, URL_SAFE_NO_PAD))
+        format!("ss://{}", URL_PASSWORD_BASE64_ENGINE.encode(param))
     }
 
     /// Get [SIP002](https://github.com/shadowsocks/shadowsocks-org/issues/27) URL
@@ -573,13 +617,13 @@ impl ServerConfig {
             if #[cfg(feature = "aead-cipher-2022")] {
                 let user_info = if !self.method().is_aead_2022() {
                     let user_info = format!("{}:{}", self.method(), self.password());
-                    encode_config(&user_info, URL_SAFE_NO_PAD)
+                    URL_PASSWORD_BASE64_ENGINE.encode(&user_info)
                 } else {
                     format!("{}:{}", self.method(), percent_encoding::utf8_percent_encode(self.password(), percent_encoding::NON_ALPHANUMERIC))
                 };
             } else {
                 let mut user_info = format!("{}:{}", self.method(), self.password());
-                user_info = encode_config(&user_info, URL_SAFE_NO_PAD)
+                user_info = URL_PASSWORD_BASE64_ENGINE.encode(&user_info)
             }
         }
 
@@ -629,7 +673,7 @@ impl ServerConfig {
                 None => return Err(UrlParseError::MissingHost),
             };
 
-            let mut decoded_body = match decode_config(encoded, URL_SAFE_NO_PAD) {
+            let mut decoded_body = match URL_PASSWORD_BASE64_ENGINE.decode(encoded) {
                 Ok(b) => match String::from_utf8(b) {
                     Ok(b) => b,
                     Err(..) => return Err(UrlParseError::InvalidServerAddr),
@@ -682,15 +726,9 @@ impl ServerConfig {
                 // reborrow to fit AsRef<[u8]>
                 let decoded_user_info: &str = &decoded_user_info;
 
-                let base64_config = if decoded_user_info.ends_with('=') {
-                    // Some implementation, like outline,
-                    // or those with Python (base64 in Python will still have '=' padding for URL safe encode)
-                    URL_SAFE
-                } else {
-                    URL_SAFE_NO_PAD
-                };
-
-                let account = match decode_config(decoded_user_info, base64_config) {
+                // Some implementation, like outline,
+                // or those with Python (base64 in Python will still have '=' padding for URL safe encode)
+                let account = match URL_PASSWORD_BASE64_ENGINE.decode(decoded_user_info) {
                     Ok(account) => match String::from_utf8(account) {
                         Ok(ac) => ac,
                         Err(..) => return Err(UrlParseError::InvalidAuthInfo),
